/*
 * This software is licensed under terms that can be found in the LICENSE file
 * in the root directory of this software component.
 *
 * Copyright (c) 2023, Georgy Moshkin <georgy.moshkin@outlook.com>
 * All rights reserved.
 *
 */

#include <iostream>
#include "uart.h"
#include "time.h"

using namespace std;

uint8_t dummy[16384];
uint8_t testData[16384];
uint8_t dataLen=0;

#define PORT_SPEED 921600



string toHex(uint8_t *data, uint8_t dataSize)
{
    char buffer [dataSize*3+1];
    buffer[dataSize*3+1-1] = 0;
    for(int j = 0; j < dataSize; j++)
        sprintf(&buffer[3*j], "%02X ", data[j]);
    return string(buffer);
}

void setXY( int x, int y )
{
    COORD coord;
    coord.X = x;
    coord.Y = y;
    SetConsoleCursorPosition(GetStdHandle( STD_OUTPUT_HANDLE ), coord);
}


int main()
{

    uint8_t devicePortNumber=0;

    for (int pn = 1; (pn <= 255) ; pn++)
    {

        if (COM_Init(0,pn,PORT_SPEED,1000))
        {

            dummy[0]=0x01;
            COM_Write((uint8_t*)&dummy,1);
            if (COM_Read((uint8_t*)&dummy,1) && (dummy[0]==0x55))
            {

                devicePortNumber=pn;
                break;
            }
            else
            {
                COM_DeInit(0);

            }
        }

    }


    if (COM_GetHandle(0)==INVALID_HANDLE_VALUE)
    {
        cout << "Device not found" << endl;
        return 0;
    }


    cout << "Port number: "<< +devicePortNumber << endl;
    cout << "Port speed: " << +PORT_SPEED << endl <<endl;

    srand (time(NULL));


    uint8_t byteTx=0;
    uint8_t byteRx=0;
    uint32_t sendGetCnt=0;
    uint32_t errorCnt=0;
    bool success=false;

    do
    {


        dummy[0]=0x02;    // byte 0: command id
        COM_Write((uint8_t*)&dummy,1);
        int32_t intTemp;
        COM_Read((uint8_t*)&intTemp,4);

        float floatTemp=intTemp/1000.0;

        setXY(0,3);
        cout.precision(2);
        cout<<"Temperature="<<fixed<<floatTemp<<endl;


        dataLen=30; //1+rand()%10; // random number from 1 to 10

        dummy[0]=0x03;    // byte 0: command id
        dummy[1]=dataLen; // byte 1: data length

        for (int i = 0; (i < dataLen) ; i++)
        {
            dummy[2+i]=rand()%256;  // byte 2 to 2+dataLen-1: random data
            testData[i]=dummy[2+i]; // copy of data for loopback test

        }

        //success=sendGet(&myPort,(uint8_t*)&dummy,2+dataLen,dataLen);
        COM_Write((uint8_t*)&dummy,2+dataLen);
        success=COM_Read((uint8_t*)&dummy,dataLen);

        for (int i = 0; (i < dataLen) ; i++)
        {
            if (dummy[i]!=testData[i]) success=false;
        }

        sendGetCnt++;


        cout << "Transfers: " << +sendGetCnt << " Errors: " << errorCnt << endl << endl;
        cout << "TX data: " << toHex(&testData[0],dataLen) << endl;
        cout << "RX data: " << toHex(&dummy[0],dataLen) << endl;



        if (success==false)
        {
            errorCnt++;
            cout << endl << "Error! Retry in 3 seconds." << endl;
            Sleep(3000);

           // while (sendGet(&myPort,(uint8_t*)&dummy,0,1)) {} // flush
           while (COM_Read((uint8_t*)&dummy,1)) {} // flush

        }


    }  while (1);


    return 0;
}

// ("<< +sendGetCnt <<"): TX=0x" << hex << +byteTx << " RX=0x" << +byteRx << dec << " sendGetResult=" << +sendGetResult << endl;
